@@ -86,6 +86,12 @@ class fulltext_native extends \phpbb\search\base
 	 */
 	protected $db;
 
+	/**
+	 * phpBB event dispatcher object
+	 * @var \phpbb\event\dispatcher_interface
+	 */
+	protected $phpbb_dispatcher;
+
 	/**
 	 * User object
 	 * @var \phpbb\user
@@ -96,13 +102,15 @@ class fulltext_native extends \phpbb\search\base
 	* Initialises the fulltext_native search backend with min/max word length and makes sure the UTF-8 normalizer is loaded
 	*
 	* @param	boolean|string	&$error	is passed by reference and should either be set to false on success or an error message on failure
+	* @param	\phpbb\event\dispatcher_interface	$phpbb_dispatcher	Event dispatcher object
 	*/
-	public function __construct(&$error, $phpbb_root_path, $phpEx, $auth, $config, $db, $user)
+	public function __construct(&$error, $phpbb_root_path, $phpEx, $auth, $config, $db, $user, $phpbb_dispatcher)
 	{
 		$this->phpbb_root_path = $phpbb_root_path;
 		$this->php_ext = $phpEx;
 		$this->config = $config;
 		$this->db = $db;
+		$this->phpbb_dispatcher = $phpbb_dispatcher;
 		$this->user = $user;
 
 		$this->word_length = array('min' => $this->config['fulltext_native_min_chars'], 'max' => $this->config['fulltext_native_max_chars']);
@@ -712,6 +720,68 @@ class fulltext_native extends \phpbb\search\base
 
 		$sql_where[] = $post_visibility;
 
+		$search_query = $this->search_query;
+		$must_exclude_one_ids = $this->must_exclude_one_ids;
+		$must_not_contain_ids = $this->must_not_contain_ids;
+		$must_contain_ids = $this->must_contain_ids;
+
+		/**
+		* Allow changing the query used for counting for posts using fulltext_native
+		*
+		* @event core.search_native_keywords_count_query_before
+		* @var	string	search_query			The parsed keywords used for this search
+		* @var	array	must_not_contain_ids	Ids that cannot be taken into account for the results
+		* @var	array	must_exclude_one_ids	Ids that cannot be on the results
+		* @var	array	must_contain_ids		Ids that must be on the results
+		* @var	int		result_count			The previous result count for the format of the query
+		*										Set to 0 to force a re-count
+		* @var	bool	join_topic				Weather or not TOPICS_TABLE should be CROSS JOIN'ED
+		* @var	array	author_ary				Array of user_id containing the users to filter the results to
+		* @var	string	author_name				An extra username to search on (!empty(author_ary) must be true, to be relevant)
+		* @var	array	ex_fid_ary				Which forums not to search on
+		* @var	int		topic_id				Limit the search to this topic_id only
+		* @var	string	sql_sort_table			Extra tables to include in the SQL query.
+		*										Used in conjunction with sql_sort_join
+		* @var	string	sql_sort_join			SQL conditions to join all the tables used together.
+		*										Used in conjunction with sql_sort_table
+		* @var	int		sort_days				Time, in days, of the oldest possible post to list
+		* @var	string	sql_where				An array of the current WHERE clause conditions
+		* @var	string	sql_match				Which columns to do the search on
+		* @var	string	sql_match_where			Extra conditions to use to properly filter the matching process
+		* @var	string	group_by				Whether or not the SQL query requires a GROUP BY for the elements in the SELECT clause
+		* @var	string	sort_by_sql				The possible predefined sort types
+		* @var	string	sort_key				The sort type used from the possible sort types
+		* @var	string	sort_dir				"a" for ASC or "d" dor DESC for the sort order used
+		* @var	string	sql_sort				The result SQL when processing sort_by_sql + sort_key + sort_dir
+		* @var	int		start					How many posts to skip in the search results (used for pagination)
+		* @since 3.1.5-RC1
+		*/
+		$vars = array(
+			'search_query',
+			'must_not_contain_ids',
+			'must_exclude_one_ids',
+			'must_contain_ids',
+			'result_count',
+			'join_topic',
+			'author_ary',
+			'author_name',
+			'ex_fid_ary',
+			'topic_id',
+			'sql_sort_table',
+			'sql_sort_join',
+			'sort_days',
+			'sql_where',
+			'sql_match',
+			'sql_match_where',
+			'group_by',
+			'sort_by_sql',
+			'sort_key',
+			'sort_dir',
+			'sql_sort',
+			'start',
+		);
+		extract($this->phpbb_dispatcher->trigger_event('core.search_native_keywords_count_query_before', compact($vars)));
+
 		if ($topic_id)
 		{
 			$sql_where[] = 'p.topic_id = ' . $topic_id;
@@ -972,6 +1042,49 @@ class fulltext_native extends \phpbb\search\base
 		$select = ($type == 'posts') ? 'p.post_id' : 't.topic_id';
 		$is_mysql = false;
 
+		/**
+		* Allow changing the query used to search for posts by author in fulltext_native
+		*
+		* @event core.search_native_author_count_query_before
+		* @var	int		total_results		The previous result count for the format of the query.
+		*									Set to 0 to force a re-count
+		* @var	string	select				SQL SELECT clause for what to get
+		* @var	string	sql_sort_table		CROSS JOIN'ed table to allow doing the sort chosen
+		* @var	string	sql_sort_join		Condition to define how to join the CROSS JOIN'ed table specifyed in sql_sort_table
+		* @var	array	sql_author			SQL WHERE condition for the post author ids
+		* @var	int		topic_id			Limit the search to this topic_id only
+		* @var	string	sort_by_sql			The possible predefined sort types
+		* @var	string	sort_key			The sort type used from the possible sort types
+		* @var	string	sort_dir			"a" for ASC or "d" dor DESC for the sort order used
+		* @var	string	sql_sort			The result SQL when processing sort_by_sql + sort_key + sort_dir
+		* @var	string	sort_days			Time, in days, that the oldest post showing can have
+		* @var	string	sql_time			The SQL to search on the time specifyed by sort_days
+		* @var	bool	firstpost_only		Wether or not to search only on the first post of the topics
+		* @var	array	ex_fid_ary			Forum ids that must not be searched on
+		* @var	array	sql_fora			SQL query for ex_fid_ary
+		* @var	int		start				How many posts to skip in the search results (used for pagination)
+		* @since 3.1.5-RC1
+		*/
+		$vars = array(
+			'total_results',
+			'select',
+			'sql_sort_table',
+			'sql_sort_join',
+			'sql_author',
+			'topic_id',
+			'sort_by_sql',
+			'sort_key',
+			'sort_dir',
+			'sql_sort',
+			'sort_days',
+			'sql_time',
+			'firstpost_only',
+			'ex_fid_ary',
+			'sql_fora',
+			'start',
+		);
+		extract($this->phpbb_dispatcher->trigger_event('core.search_native_author_count_query_before', compact($vars)));
+
 		// If the cache was completely empty count the results
 		if (!$total_results)
 		{
@@ -1075,7 +1188,7 @@ class fulltext_native extends \phpbb\search\base
 			// Count rows for the executed queries. Replace $select within $sql with SQL_CALC_FOUND_ROWS, and run it.
 			$sql_calc = str_replace('SELECT ' . $select, 'SELECT DISTINCT SQL_CALC_FOUND_ROWS p.post_id', $sql);
 
-			$this->db->sql_query($sql_calc);
+			$result = $this->db->sql_query($sql_calc);
 			$this->db->sql_freeresult($result);
 
 			$sql_count = 'SELECT FOUND_ROWS() as total_results';
